@@ -1,4 +1,4 @@
-# $Id: LibXML.pm,v 1.59 2002/05/20 10:36:49 phish Exp $
+# $Id: LibXML.pm,v 1.66 2002/05/29 20:52:06 phish Exp $
 
 package XML::LibXML;
 
@@ -10,7 +10,7 @@ use Carp;
 use XML::LibXML::NodeList;
 use IO::Handle; # for FH reads called as methods
 
-$VERSION = "1.50";
+$VERSION = "1.51";
 require Exporter;
 require DynaLoader;
 
@@ -511,7 +511,14 @@ sub findnodes {
 
 sub findvalue {
     my ($node, $xpath) = @_;
-    return $node->find($xpath)->to_literal->value;
+    my $res;
+    eval {
+        $res = $node->find($xpath);
+    };
+    if  ( $@ ) {
+        die $@;
+    }
+    return $res->to_literal->value;
 }
 
 sub find {
@@ -582,11 +589,10 @@ use vars qw(@ISA);
 
 sub toString {
     my $self = shift;
-    my $enc  = shift;
     my $retval = "";
     if ( $self->hasChildNodes() ) {
         foreach my $n ( $self->childNodes() ) {
-            $retval .= $n->toString($enc);
+            $retval .= $n->toString(@_);
         }
     }
     return $retval;
@@ -786,12 +792,16 @@ package XML::LibXML::Namespace;
 # this is infact not a node!
 sub prefix { return "xmlns"; }
 
+sub getNamespaces { return (); }
+
 sub nodeName {
     my $self = shift;
     my $nsP  = $self->name;
     return length($nsP) ? "xmlns:$nsP" : "xmlns";
 }
 
+sub getNodeName { my $self = shift; return $self->nodeName; }
+
 sub isEqualNode {
     my ( $self, $ref ) = @_;
     if ( ref($ref) eq "XML::LibXML::Namespace" ) {
@@ -901,19 +911,16 @@ sub removeNamedItemNS {
 package XML::LibXML::_SAXParser;
 
 # this is pseudo class!!!
-use Carp;
+
+use XML::SAX::Exception;
+
+# NOTE: there is not end_document ON PURPOSE!
 
 sub start_document {
     my $parser = shift;
-    $parser->{SAX}->{State} = 1;
     $parser->{HANDLER}->start_document({});
 }
 
-sub end_document {
-    my $parser = shift;
-    $parser->{SAX}->{State} = 0;
-}
-
 sub xml_decl {
     my ( $parser, $version, $encoding ) = @_;
 
@@ -922,47 +929,87 @@ sub xml_decl {
     $parser->{HANDLER}->xml_decl($decl);
 }
 
+sub start_prefix_mapping {
+    my ( $parser, $prefix, $uri ) = @_;
+    $parser->{HANDLER}->start_prefix_mapping( { Prefix => $prefix, NamespaceURI => $uri } );
+}
+
+sub end_prefix_mapping {
+    my ( $parser, $prefix, $uri ) = @_;
+    $parser->{HANDLER}->end_prefix_mapping( { Prefix => $prefix, NamespaceURI => $uri } );
+}
+
 sub start_element {
     my (  $parser, $elem, $attrs ) = @_;
     my $saxattr = {};
 
     push @{$parser->{SAX}->{ELSTACK}}, $elem;
-    if ( defined $attrs ) {
+
+    if ( defined $attrs  ) {
         $parser->{HANDLER}->start_element( { %$elem, Attributes=>$attrs} )
     }
-
+    else {
+        $parser->{HANDLER}->start_element( $elem )
+    }
 }
 
 sub end_element {
     my (  $parser, $name ) = @_;
     my $elem = pop @{$parser->{SAX}->{ELSTACK}};
     if ( $elem->{Name} ne $name ) {
-        croak( "cought error where parser should work ($elem->{Name} != $name" );
+        my $error = XML::SAX::Execption::Parse->new( Message => "cought error where parser should catch ('$elem->{Name}' ne '$name' )" );
+        $parser->{HANDLER}->error( $error );
+        return;
     }
     $parser->{HANDLER}->end_element( $elem );
 }
 
 sub characters {
     my ( $parser, $data ) = @_;
-    $parser->{HANDLER}->characters( {Data => $data} );
+    $parser->{HANDLER}->characters( $data );
 }
 
 sub comment {
     my ( $parser, $data ) = @_;
-    $parser->{HANDLER}->comment( {Data => $data} );
+    $parser->{HANDLER}->comment( $data );
 }
 
 sub cdata_block {
     my ( $parser, $data ) = @_;
     $parser->{HANDLER}->start_cdata();
-    $parser->{HANDLER}->characters( {Data => $data} );
+    $parser->{HANDLER}->characters( $data );
     $parser->{HANDLER}->end_cdata();
 }
 
 sub processing_instruction {
-    my ( $parser, $target, $data ) = @_;
-    $parser->{HANDLER}->processing_instruction( {Target => $target,
-                                                 Data   => $data} );
+    my ( $parser, $target ) = @_;
+    $parser->{HANDLER}->processing_instruction( $target );
+}
+
+# these functions will use SAX exceptions as soon i know how things really work
+sub warning {
+    my ( $parser, $message, $line, $col ) = @_;
+    my $error = XML::SAX::Exception::Parse->new( LineNumber   => $line,
+                                                 ColumnNumber => $col,
+                                                 Message      => $message, );
+    $parser->{HANDLER}->warning( $error );
+}
+
+sub error {
+    my ( $parser, $message, $line, $col ) = @_;
+
+    my $error = XML::SAX::Exception::Parse->new( LineNumber   => $line,
+                                                 ColumnNumber => $col,
+                                                 Message      => $message, );
+    $parser->{HANDLER}->error( $error );
+}
+
+sub fatal_error {
+    my ( $parser, $message, $line, $col ) = @_;
+    my $error = XML::SAX::Exception::Parse->new( LineNumber   => $line,
+                                                 ColumnNumber => $col,
+                                                 Message      => $message, );
+    $parser->{HANDLER}->fatal_error( $error );
 }
 
 1;
@@ -1127,7 +1174,7 @@ parsing.
 
   my $doc = $parser->parse_file($filename);
 
-=head1 PARSING HTML
+=head2 Parsing Html
 
 As of version 0.96, XML::LibXML is capable of parsing HTML into a
 regular XML DOM. This gives you the full power of XML::LibXML on HTML
@@ -1150,21 +1197,7 @@ to do that.
 
   my $doc = $parser->parse_html_file($filename);
 
-=head2 Extra parsing methods
-
-B<processXIncludes>
-
-  $parser->processXIncludes( $doc );
-
-While the document class implements a separate XInclude processing,
-this method, is stricly related to the parser. The use of this method
-is only required, if the parser implements special callbacks that
-should to be used for the XInclude as well.
-
-If expand_xincludes is set to 1, the method is only required to process
-XIncludes appended to the DOM after its original parsing.
-
-=head1 PUSH PARSER
+=head2 Push Parser
 
 XML::LibXML supports also a push parser interface. This allows one to
 parse large documents without actually loading the entire document
@@ -1218,7 +1251,42 @@ the parser before.
 
 =back
 
-=head1 SERIALIZATION
+=head2 Extra parsing methods
+
+B<processXIncludes>
+
+  $parser->processXIncludes( $doc );
+
+While the document class implements a separate XInclude processing,
+this method, is stricly related to the parser. The use of this method
+is only required, if the parser implements special callbacks that
+should to be used for the XInclude as well.
+
+If expand_xincludes is set to 1, the method is only required to process
+XIncludes appended to the DOM after its original parsing.
+
+=head2 Error Handling
+
+XML::LibXML throws exceptions during parseing, validation or XPath
+processing. These errors can be catched by useing eval blocks. The
+error then will be stored in B<$@>. Alternatively one can use the
+get_last_error() function of XML::LibXML. It will return the same
+string that is stored in $@. Using get_last_error() makes it still
+nessecary to eval the statement, since these function groups will
+die() on errors.
+
+get_last_error() can be called either by the class itself or by a
+parser instance:
+
+   $errstring = XML::LibXML->get_last_error();
+   $errstring = $parser->get_last_error();
+
+Note that XML::LibXML exceptions are global. That means if
+get_last_error is called on an parser instance, the last B<global>
+error will be returned. This is not nessecarily the error caused by
+the parser instance itself.
+
+=head2 Serialization
 
 The oposite of parsing is serialization. In XML::LibXML this can be
 done by using the functions toString(), toFile() and toFH(). All
@@ -1255,30 +1323,11 @@ toString() for the node. Since a node has no DTD and no XML
 Declaration the related flags will take no effect. Nevertheless
 setTagCompression is supported.
 
-=head1 XML::LibXML::Document
+All basic serialization function recognize an additional formating
+flag. This flag is an easy way to format complex xml documents without
+adding ignoreable whitespaces.
 
-The objects returned above have a few methods available to them:
-
-=head2 C<$doc-E<gt>toString>
-
-Convert the document to a string.
-
-=head2 C<$doc-E<gt>is_valid>
-
-Post parse validation. Returns true if the document is valid against the
-DTD specified in the DOCTYPE declaration
-
-=head2 C<$doc-E<gt>is_valid($dtd)>
-
-Same as the above, but allows you to pass in a DTD created from
-L<"XML::LibXML::Dtd">.
-
-=head2 C<$doc-E<gt>process_xinclude>
-
-Process any xinclude tags in the file. (currently using B<only> libxml2's
-default callbacks)
-
-=head1 Input Callbacks
+=head2 Input Callbacks
 
 The input callbacks are used whenever LibXML has to get something B<other
 than external parsed entities> from somewhere. The input callbacks in LibXML
@@ -1375,7 +1424,7 @@ functions directly on the class.
 The previous code snippet will set the callbacks from the first
 example as global callbacks.
 
-=head1 Encoding
+=head2 Encoding
 
 All data will be stored UTF-8 encoded. Nevertheless the input and
 output functions are aware about the encoding of the owner
